@@ -9,7 +9,9 @@
 #include <algorithm>
 #include <regex>
 #include <inttypes.h>
+#include <spdlog/spdlog.h>
 #include "string_utils.h"
+#include "gpu.h"
 
 #ifndef PROCDIR
 #define PROCDIR "/proc"
@@ -23,10 +25,6 @@
 #define PROCMEMINFOFILE PROCDIR "/meminfo"
 #endif
 
-#ifndef PROCCPUINFOFILE
-#define PROCCPUINFOFILE PROCDIR "/cpuinfo"
-#endif
-
 #include "file_utils.h"
 
 void calculateCPUData(CPUData& cpuData,
@@ -116,13 +114,13 @@ bool CPUStats::Init()
     m_cpuData.clear();
 
     if (!file.is_open()) {
-        std::cerr << "Failed to opening " << PROCSTATFILE << std::endl;
+        SPDLOG_ERROR("Failed to opening " PROCSTATFILE);
         return false;
     }
 
     do {
         if (!std::getline(file, line)) {
-            std::cerr << "Failed to read all of " << PROCSTATFILE << std::endl;
+            SPDLOG_DEBUG("Failed to read all of " PROCSTATFILE);
             return false;
         } else if (starts_with(line, "cpu")) {
             if (first) {
@@ -133,6 +131,7 @@ bool CPUStats::Init()
             CPUData cpu = {};
             cpu.totalTime = 1;
             cpu.totalPeriod = 1;
+            sscanf(line.c_str(), "cpu%4d ", &cpu.cpu_id);
             m_cpuData.push_back(cpu);
 
         } else if (starts_with(line, "btime ")) {
@@ -175,7 +174,7 @@ bool CPUStats::UpdateCPUData()
     bool ret = false;
 
     if (!file.is_open()) {
-        std::cerr << "Failed to opening " << PROCSTATFILE << std::endl;
+        SPDLOG_ERROR("Failed to opening " PROCSTATFILE);
         return false;
     }
 
@@ -189,25 +188,24 @@ bool CPUStats::UpdateCPUData()
         } else if (sscanf(line.c_str(), "cpu%4d %16llu %16llu %16llu %16llu %16llu %16llu %16llu %16llu %16llu %16llu",
             &cpuid, &usertime, &nicetime, &systemtime, &idletime, &ioWait, &irq, &softIrq, &steal, &guest, &guestnice) == 11) {
 
-            //std::cerr << "Parsing 'cpu" << cpuid << "' line:" <<  line << std::endl;
+            //SPDLOG_DEBUG("Parsing 'cpu{}' line:{}", cpuid, line);
 
             if (!ret) {
-                //std::cerr << "Failed to parse 'cpu' line" << std::endl;
-                std::cerr << "Failed to parse 'cpu' line:" <<  line << std::endl;
+                SPDLOG_DEBUG("Failed to parse 'cpu' line:{}", line);
                 return false;
             }
 
             if (cpuid < 0 /* can it? */) {
-                std::cerr << "Cpu id '" << cpuid << "' is out of bounds" << std::endl;
+                SPDLOG_DEBUG("Cpu id '{}' is out of bounds", cpuid);
                 return false;
             }
 
-            if ((size_t)cpuid >= m_cpuData.size()) {
-                std::cerr << "Cpu id '" << cpuid << "' is out of bounds, reiniting" << std::endl;
+            if (cpu_count + 1 > m_cpuData.size() || m_cpuData[cpu_count].cpu_id != cpuid) {
+                SPDLOG_DEBUG("Cpu id '{}' is out of bounds or wrong index, reiniting", cpuid);
                 return Reinit();
             }
 
-            CPUData& cpuData = m_cpuData[cpuid];
+            CPUData& cpuData = m_cpuData[cpu_count];
             calculateCPUData(cpuData, usertime, nicetime, systemtime, idletime, ioWait, irq, softIrq, steal, guest, guestnice);
             cpuid = -1;
             cpu_count++;
@@ -227,38 +225,46 @@ bool CPUStats::UpdateCPUData()
 
 bool CPUStats::UpdateCoreMhz() {
     m_coreMhz.clear();
-    std::ifstream cpuInfo(PROCCPUINFOFILE);
-    std::string row;
-    size_t i = 0;
-    while (std::getline(cpuInfo, row) && i < m_cpuData.size()) {
-        if (row.find("MHz") != std::string::npos){
-            row = std::regex_replace(row, std::regex(R"([^0-9.])"), "");
-            if (!try_stoi(m_cpuData[i].mhz, row))
-                m_cpuData[i].mhz = 0;
-            i++;
+    FILE *fp;
+    for (auto& cpu : m_cpuData)
+    {
+        std::string path = "/sys/devices/system/cpu/cpu" + std::to_string(cpu.cpu_id) + "/cpufreq/scaling_cur_freq";
+        if ((fp = fopen(path.c_str(), "r"))){
+            int64_t temp;
+            if (fscanf(fp, "%" PRId64, &temp) != 1)
+                temp = 0;
+            cpu.mhz = temp / 1000;
+            fclose(fp);
         }
     }
+
     m_cpuDataTotal.cpu_mhz = 0;
     for (auto data : m_cpuData)
-        m_cpuDataTotal.cpu_mhz += data.mhz;
-    m_cpuDataTotal.cpu_mhz /= m_cpuData.size();
+        if (data.mhz > m_cpuDataTotal.cpu_mhz)
+            m_cpuDataTotal.cpu_mhz = data.mhz;
+
     return true;
 }
 
 bool CPUStats::UpdateCpuTemp() {
-    if (!m_cpuTempFile)
-        return false;
+    if (cpu_type == "APU"){
+        m_cpuDataTotal.temp = gpu_info.apu_cpu_temp;
+        return true;
+    } else {
+        if (!m_cpuTempFile)
+            return false;
 
-    int temp = 0;
-    rewind(m_cpuTempFile);
-    fflush(m_cpuTempFile);
-    bool ret = (fscanf(m_cpuTempFile, "%d", &temp) == 1);
-    m_cpuDataTotal.temp = temp / 1000;
+        int temp = 0;
+        rewind(m_cpuTempFile);
+        fflush(m_cpuTempFile);
+        bool ret = (fscanf(m_cpuTempFile, "%d", &temp) == 1);
+        m_cpuDataTotal.temp = temp / 1000;
 
-    return ret;
+        return ret;
+    }
 }
 
-static bool get_cpu_power_k10temp(CPUPowerData* cpuPowerData, int& power) {
+static bool get_cpu_power_k10temp(CPUPowerData* cpuPowerData, float& power) {
     CPUPowerData_k10temp* powerData_k10temp = (CPUPowerData_k10temp*)cpuPowerData;
 
     if (!powerData_k10temp->coreVoltageFile || !powerData_k10temp->coreCurrentFile || !powerData_k10temp->socVoltageFile || !powerData_k10temp->socCurrentFile)
@@ -291,7 +297,7 @@ static bool get_cpu_power_k10temp(CPUPowerData* cpuPowerData, int& power) {
     return true;
 }
 
-static bool get_cpu_power_zenpower(CPUPowerData* cpuPowerData, int& power) {
+static bool get_cpu_power_zenpower(CPUPowerData* cpuPowerData, float& power) {
     CPUPowerData_zenpower* powerData_zenpower = (CPUPowerData_zenpower*)cpuPowerData;
 
     if (!powerData_zenpower->corePowerFile || !powerData_zenpower->socPowerFile)
@@ -315,7 +321,7 @@ static bool get_cpu_power_zenpower(CPUPowerData* cpuPowerData, int& power) {
     return true;
 }
 
-static bool get_cpu_power_rapl(CPUPowerData* cpuPowerData, int& power) {
+static bool get_cpu_power_rapl(CPUPowerData* cpuPowerData, float& power) {
     CPUPowerData_rapl* powerData_rapl = (CPUPowerData_rapl*)cpuPowerData;
 
     if (!powerData_rapl->energyCounterFile)
@@ -342,11 +348,16 @@ static bool get_cpu_power_rapl(CPUPowerData* cpuPowerData, int& power) {
     return true;
 }
 
+static bool get_cpu_power_amdgpu(float& power) {
+    power = gpu_info.apu_cpu_power;
+    return true;
+}
+
 bool CPUStats::UpdateCpuPower() {
     if(!m_cpuPowerData)
         return false;
 
-    int power = 0;
+    float power = 0;
 
     switch(m_cpuPowerData->source) {
         case CPU_POWER_K10TEMP:
@@ -358,6 +369,9 @@ bool CPUStats::UpdateCpuPower() {
         case CPU_POWER_RAPL:
             if (!get_cpu_power_rapl(m_cpuPowerData.get(), power)) return false;
             break;
+        case CPU_POWER_AMDGPU:
+            if (!get_cpu_power_amdgpu(power)) return false;
+            break;
         default:
             return false;
     }
@@ -399,9 +413,7 @@ static bool find_fallback_temp_input(const std::string path, std::string& input)
         if (!ends_with(file, "_input"))
             continue;
         input = path + "/" + file;
-#ifndef NDEBUG
-        std::cerr << "fallback cpu temp input: " << input << "\n";
-#endif
+        SPDLOG_DEBUG("fallback cpu temp input: {}", input);
         return true;
     }
     return false;
@@ -418,9 +430,8 @@ bool CPUStats::GetCpuFile() {
     for (auto& dir : dirs) {
         path = hwmon + dir;
         name = read_line(path + "/name");
-#ifndef NDEBUG
-        std::cerr << "hwmon: sensor name: " << name << std::endl;
-#endif
+        SPDLOG_DEBUG("hwmon: sensor name: {}", name);
+
         if (name == "coretemp") {
             find_temp_input(path, input, "Package id 0");
             break;
@@ -435,14 +446,11 @@ bool CPUStats::GetCpuFile() {
             path.clear();
         }
     }
-
     if (path.empty() || (!file_exists(input) && !find_fallback_temp_input(path, input))) {
-        std::cerr << "MANGOHUD: Could not find cpu temp sensor location" << std::endl;
+        SPDLOG_ERROR("Could not find cpu temp sensor location");
         return false;
     } else {
-#ifndef NDEBUG
-        std::cerr << "hwmon: using input: " << input << std::endl;
-#endif
+        SPDLOG_DEBUG("hwmon: using input: {}", input);
         m_cpuTempFile = fopen(input.c_str(), "r");
     }
     return true;
@@ -480,12 +488,10 @@ CPUPowerData_k10temp* init_cpu_power_data_k10temp(const std::string path) {
     if(!find_input(path, "in", socVoltageInput, "Vsoc")) return nullptr;
     if(!find_input(path, "curr", socCurrentInput, "Isoc")) return nullptr;
 
-#ifndef NDEBUG
-    std::cerr << "hwmon: using input: " << coreVoltageInput << std::endl;
-    std::cerr << "hwmon: using input: " << coreCurrentInput << std::endl;
-    std::cerr << "hwmon: using input: " << socVoltageInput << std::endl;
-    std::cerr << "hwmon: using input: " << socCurrentInput << std::endl;
-#endif
+    SPDLOG_DEBUG("hwmon: using input: {}", coreVoltageInput);
+    SPDLOG_DEBUG("hwmon: using input: {}", coreCurrentInput);
+    SPDLOG_DEBUG("hwmon: using input: {}", socVoltageInput);
+    SPDLOG_DEBUG("hwmon: using input: {}", socCurrentInput);
 
     powerData->coreVoltageFile = fopen(coreVoltageInput.c_str(), "r");
     powerData->coreCurrentFile = fopen(coreCurrentInput.c_str(), "r");
@@ -503,10 +509,8 @@ CPUPowerData_zenpower* init_cpu_power_data_zenpower(const std::string path) {
     if(!find_input(path, "power", corePowerInput, "SVI2_P_Core")) return nullptr;
     if(!find_input(path, "power", socPowerInput, "SVI2_P_SoC")) return nullptr;
 
-#ifndef NDEBUG
-    std::cerr << "hwmon: using input: " << corePowerInput << std::endl;
-    std::cerr << "hwmon: using input: " << socPowerInput << std::endl;
-#endif
+    SPDLOG_DEBUG("hwmon: using input: {}", corePowerInput);
+    SPDLOG_DEBUG("hwmon: using input: {}", socPowerInput);
 
     powerData->corePowerFile = fopen(corePowerInput.c_str(), "r");
     powerData->socPowerFile = fopen(socPowerInput.c_str(), "r");
@@ -531,6 +535,7 @@ bool CPUStats::InitCpuPowerData() {
 
     std::string name, path;
     std::string hwmon = "/sys/class/hwmon/";
+    bool intel = false;
 
     CPUPowerData* cpuPowerData = nullptr;
 
@@ -538,36 +543,39 @@ bool CPUStats::InitCpuPowerData() {
     for (auto& dir : dirs) {
         path = hwmon + dir;
         name = read_line(path + "/name");
-#ifndef NDEBUG
-        std::cerr << "hwmon: sensor name: " << name << std::endl;
-#endif
+        SPDLOG_DEBUG("hwmon: sensor name: {}", name);
+
         if (name == "k10temp") {
             cpuPowerData = (CPUPowerData*)init_cpu_power_data_k10temp(path);
             break;
         } else if (name == "zenpower") {
             cpuPowerData = (CPUPowerData*)init_cpu_power_data_zenpower(path);
             break;
+        } else if (name == "coretemp") {
+            intel = true;
         }
     }
 
-    if (!cpuPowerData) {
+    if (!cpuPowerData && intel) {
         std::string powercap = "/sys/class/powercap/";
         auto powercap_dirs = ls(powercap.c_str());
         for (auto& dir : powercap_dirs) {
             path = powercap + dir;
             name = read_line(path + "/name");
-#ifndef NDEBUG
-            std::cerr << "powercap: name: " << name << std::endl;
-#endif
+            SPDLOG_DEBUG("powercap: name: {}", name);
             if (name == "package-0") {
                 cpuPowerData = (CPUPowerData*)init_cpu_power_data_rapl(path);
                 break;
             }
         }
     }
+    if (!cpuPowerData && !intel) {
+        auto powerData = std::make_unique<CPUPowerData_amdgpu>();
+        cpuPowerData = (CPUPowerData*)powerData.release();
+    }
 
     if(cpuPowerData == nullptr) {
-        std::cerr << "MANGOHUD: Failed to initialize CPU power data" << std::endl;
+        SPDLOG_ERROR("Failed to initialize CPU power data");
         return false;
     }
 
