@@ -4,6 +4,7 @@
 
 #include <cstdio>
 #include <cstdint>
+#include "overlay_params.h"
 
 struct amdgpu_files
 {
@@ -15,6 +16,7 @@ struct amdgpu_files
     FILE *core_clock;
     FILE *memory_clock;
     FILE *power_usage;
+    FILE *gtt_used;
 };
 
 extern amdgpu_files amdgpu;
@@ -33,13 +35,13 @@ struct gpuInfo{
     bool is_current_throttled;
     bool is_temp_throttled;
     bool is_other_throttled;
+    float gtt_used;
 };
 
 extern struct gpuInfo gpu_info;
 
-void getNvidiaGpuInfo(void);
+void getNvidiaGpuInfo(const struct overlay_params& params);
 void getAmdGpuInfo(void);
-extern decltype(&getAmdGpuInfo) getAmdGpuInfo_actual;
 bool checkNvidia(const char *pci_dev);
 extern void nvapi_util();
 extern bool checkNVAPI();
