@@ -21,6 +21,8 @@
 #define CHAR_CELSIUS    "\xe2\x84\x83"
 #define CHAR_FAHRENHEIT "\xe2\x84\x89"
 
+using namespace std;
+
 // Cut from https://github.com/ocornut/imgui/pull/2943
 // Probably move to ImGui
 float SRGBToLinear(float in)
@@ -120,6 +122,26 @@ void HudElements::convert_colors(bool do_conv, const struct overlay_params& para
     convert_colors(params);
 }
 
+/**
+* Go to next column or second column on new row
+*/
+void ImguiNextColumnOrNewRow(int column = -1)
+{
+    if (column > -1 && column < ImGui::TableGetColumnCount())
+        ImGui::TableSetColumnIndex(column);
+    else
+    {
+        ImGui::TableNextColumn();
+        if (ImGui::TableGetColumnIndex() == 0 && ImGui::TableGetColumnCount() > 1)
+            ImGui::TableNextColumn();
+    }
+}
+
+void ImGuiTableSetColumnIndex(int column)
+{
+    ImGui::TableSetColumnIndex(std::max(0, std::min(column, ImGui::TableGetColumnCount() - 1)));
+}
+
 void HudElements::time(){
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_time]){
         ImGui::TableNextRow(); ImGui::TableNextColumn();
@@ -167,16 +189,13 @@ void HudElements::gpu_stats(){
             // ImGui::Text("%s", "%");
         }
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_temp]){
-            ImGui::TableNextColumn();
+            ImguiNextColumnOrNewRow();
             right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu_info.temp);
             ImGui::SameLine(0, 1.0f);
             ImGui::Text("°C");
         }
-        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_core_clock] || HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_power]){
-            ImGui::TableNextRow(); ImGui::TableNextColumn();
-        }
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_core_clock]){
-            ImGui::TableNextColumn();
+            ImguiNextColumnOrNewRow();
             right_aligned_text(text_color, HUDElements.ralign_width, "%i", gpu_info.CoreClock);
             ImGui::SameLine(0, 1.0f);
             ImGui::PushFont(HUDElements.sw_stats->font1);
@@ -184,7 +203,7 @@ void HudElements::gpu_stats(){
             ImGui::PopFont();
         }
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_power]) {
-            ImGui::TableNextColumn();
+            ImguiNextColumnOrNewRow();
 #ifdef MANGOAPP
             right_aligned_text(text_color, HUDElements.ralign_width, "%.1f", gpu_info.powerUsage);
 #else
@@ -232,24 +251,23 @@ void HudElements::cpu_stats(){
         }
 
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_cpu_temp]){
-            ImGui::TableNextColumn();
+            ImguiNextColumnOrNewRow();
             right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", cpuStats.GetCPUDataTotal().temp);
             ImGui::SameLine(0, 1.0f);
             ImGui::Text("°C");
         }
-        if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_cpu_mhz] || HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_cpu_power]){
-            ImGui::TableNextRow(); ImGui::TableNextColumn();
-        }
+
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_cpu_mhz]){
-            ImGui::TableNextColumn();
+            ImguiNextColumnOrNewRow();
             right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", cpuStats.GetCPUDataTotal().cpu_mhz);
             ImGui::SameLine(0, 1.0f);
             ImGui::PushFont(HUDElements.sw_stats->font1);
             ImGui::Text("MHz");
             ImGui::PopFont();
         }
+
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_cpu_power]){
-            ImGui::TableNextColumn();
+            ImguiNextColumnOrNewRow();
         #ifdef MANGOAPP
             right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", cpuStats.GetCPUDataTotal().power);
         #else
@@ -288,13 +306,13 @@ void HudElements::core_load(){
                 right_aligned_text(load_color, HUDElements.ralign_width, "%d", cpu_load_percent);
                 ImGui::SameLine(0, 1.0f);
                 ImGui::TextColored(load_color, "%%");
-                ImGui::TableNextColumn();
+                ImguiNextColumnOrNewRow();
             }
             else {
                 right_aligned_text(text_color, HUDElements.ralign_width, "%i", int(cpuData.percent));
                 ImGui::SameLine(0, 1.0f);
                 ImGui::Text("%%");
-                ImGui::TableNextColumn();
+                ImguiNextColumnOrNewRow();
             }
             right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", cpuData.mhz);
             ImGui::SameLine(0, 1.0f);
@@ -326,7 +344,7 @@ void HudElements::io_stats(){
             ImGui::PopFont();
         }
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_io_write]){
-            ImGui::TableNextColumn();
+            ImguiNextColumnOrNewRow();
             const float val = g_io_stats.per_second.write;
             right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, val < 100 ? "%.1f" : "%.f", val);
             ImGui::SameLine(0,1.0f);
@@ -343,14 +361,18 @@ void HudElements::vram(){
         ImGui::TableNextRow(); ImGui::TableNextColumn();
         ImGui::TextColored(HUDElements.colors.vram, "VRAM");
         ImGui::TableNextColumn();
-        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", gpu_info.memoryUsed);
+        // Add gtt_used to vram usage for APUs
+        if (cpuStats.cpu_type == "APU")
+            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", gpu_info.memoryUsed + gpu_info.gtt_used);
+        else
+            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", gpu_info.memoryUsed);
         ImGui::SameLine(0,1.0f);
         ImGui::PushFont(HUDElements.sw_stats->font1);
         ImGui::Text("GiB");
         ImGui::PopFont();
 #ifndef MANGOAPP
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gpu_mem_clock]){
-            ImGui::TableNextColumn();
+            ImguiNextColumnOrNewRow();
             right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", gpu_info.MemClock);
             ImGui::SameLine(0, 1.0f);
             ImGui::PushFont(HUDElements.sw_stats->font1);
@@ -375,7 +397,7 @@ void HudElements::ram(){
     }
 
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_ram] && HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_swap]){
-        ImGui::TableNextColumn();
+        ImguiNextColumnOrNewRow();
         right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", swapused);
         ImGui::SameLine(0,1.0f);
         ImGui::PushFont(HUDElements.sw_stats->font1);
@@ -402,7 +424,7 @@ void HudElements::procmem()
     ImGui::PopFont();
 
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_procmem_shared]){
-        ImGui::TableNextColumn();
+        ImguiNextColumnOrNewRow();
         right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", format_units(proc_mem.shared, unit));
         ImGui::SameLine(0,1.0f);
         ImGui::PushFont(HUDElements.sw_stats->font1);
@@ -410,13 +432,8 @@ void HudElements::procmem()
         ImGui::PopFont();
     }
 
-    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_procmem_shared] && HUDElements.params->table_columns < 4){
-        ImGui::TableNextRow();
-        ImGui::TableNextColumn();
-    }
-
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_procmem_virt]){
-        ImGui::TableNextColumn();
+        ImguiNextColumnOrNewRow();
         right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", format_units(proc_mem.virt, unit));
         ImGui::SameLine(0,1.0f);
         ImGui::PushFont(HUDElements.sw_stats->font1);
@@ -451,7 +468,7 @@ void HudElements::fps(){
         ImGui::Text("FPS");
         ImGui::PopFont();
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_frametime]){
-            ImGui::TableNextColumn();
+            ImguiNextColumnOrNewRow();
             right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", 1000 / HUDElements.sw_stats->fps);
             ImGui::SameLine(0, 1.0f);
             ImGui::PushFont(HUDElements.sw_stats->font1);
@@ -466,8 +483,9 @@ void HudElements::fps(){
 
 void HudElements::fps_only(){
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_fps_only]){
+        ImGui::TableNextRow(); ImGui::TableNextColumn();
+        auto load_color = HUDElements.colors.text;
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_fps_color_change]){
-            ImGui::TableNextRow(); ImGui::TableNextColumn();
             int fps = int(HUDElements.sw_stats->fps);
             struct LOAD_DATA fps_data = {
             HUDElements.colors.fps_value_low,
@@ -476,13 +494,9 @@ void HudElements::fps_only(){
             HUDElements.params->fps_value[0],
             HUDElements.params->fps_value[1]
             };
-            auto load_color = change_on_load_temp(fps_data, fps);
-            ImGui::TextColored(load_color, "%.0f", HUDElements.sw_stats->fps);
-        }
-        else {
-            ImGui::TableNextRow(); ImGui::TableNextColumn();
-            ImGui::TextColored(HUDElements.colors.text, "%.0f", HUDElements.sw_stats->fps);
+            load_color = change_on_load_temp(fps_data, fps);
         }
+        ImGui::TextColored(load_color, "%.0f", HUDElements.sw_stats->fps);
     }
 }
 
@@ -559,11 +573,9 @@ void HudElements::frame_timing(){
         ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
         ImGui::PushFont(HUDElements.sw_stats->font1);
         ImGui::TextColored(HUDElements.colors.engine, "%s", "Frametime");
-        for (size_t i = 0; i < HUDElements.params->table_columns - 1; i++)
-            ImGui::TableNextColumn();
+        ImGui::TableSetColumnIndex(ImGui::TableGetColumnCount() - 1);
         ImGui::Dummy(ImVec2(0.0f, real_font_size.y));
-        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width * 1.3, "min: %.1fms, max: %.1fms", min_frametime, max_frametime);
-        ImGui::PopFont();
+        right_aligned_text(HUDElements.colors.text, ImGui::GetContentRegionAvail().x, "min: %.1fms, max: %.1fms", min_frametime, max_frametime);
         ImGui::TableNextRow(); ImGui::TableNextColumn();
         char hash[40];
         snprintf(hash, sizeof(hash), "##%s", overlay_param_names[OVERLAY_PARAM_ENABLED_frame_timing]);
@@ -572,28 +584,29 @@ void HudElements::frame_timing(){
         ImGui::PushStyleColor(ImGuiCol_FrameBg, ImVec4(0.0f, 0.0f, 0.0f, 0.0f));
         double min_time = 0.0f;
         double max_time = 50.0f;
+#ifdef MANGOAPP
+        const ImVec2 sz = ImGui::CalcTextSize("1000.0ms");
+        float width = ImGui::GetWindowContentRegionWidth() - sz.x;
+#else
+        float width = ImGui::GetWindowContentRegionWidth();
+#endif
         if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_histogram]){
             ImGui::PlotHistogram(hash, get_time_stat, HUDElements.sw_stats,
                                 ARRAY_SIZE(HUDElements.sw_stats->frames_stats), 0,
                                 NULL, min_time, max_time,
-                                ImVec2(ImGui::GetContentRegionAvailWidth() * HUDElements.params->table_columns, 50));
+                                ImVec2(width, 50));
         } else {
-#ifdef MANGOAPP
-        int width = ImGui::GetContentRegionAvailWidth() * HUDElements.params->table_columns - 30;
-#else
-        int width = ImGui::GetContentRegionAvailWidth() * HUDElements.params->table_columns;
-#endif
             ImGui::PlotLines(hash, get_time_stat, HUDElements.sw_stats,
                             ARRAY_SIZE(HUDElements.sw_stats->frames_stats), 0,
                             NULL, min_time, max_time,
                             ImVec2(width, 50));
-#ifdef MANGOAPP
-            ImGui::SameLine();
-            ImGui::PushFont(HUDElements.sw_stats->font1);
-            ImGui::Text("%.1fms", frametime / 1000.f);
-            ImGui::PopFont();
-#endif
         }
+#ifdef MANGOAPP
+        ImGui::SameLine();
+        ImGuiTableSetColumnIndex(ImGui::TableGetColumnCount() - 1);
+        right_aligned_text(HUDElements.colors.text, ImGui::GetContentRegionAvail().x, "%.1fms", frametime / 1000.f);
+#endif
+        ImGui::PopFont();
         ImGui::PopStyleColor();
     }
 }
@@ -610,8 +623,11 @@ void HudElements::media_player(){
     scaled_font.Scale = HUDElements.params->font_scale_media_player;
     ImGui::PushFont(&scaled_font);
     {
-        std::lock_guard<std::mutex> lck(main_metadata.mtx);
-        render_mpris_metadata(*HUDElements.params, main_metadata, frame_timing);
+        std::unique_lock<std::mutex> lck(main_metadata.mtx, std::try_to_lock);
+        if (lck.owns_lock())
+            render_mpris_metadata(*HUDElements.params, main_metadata, frame_timing);
+        else
+            SPDLOG_DEBUG("failed to acquire lock");
     }
     ImGui::PopFont();
 #endif
@@ -620,12 +636,11 @@ void HudElements::media_player(){
 void HudElements::resolution(){
     if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_resolution]){
         ImGui::TableNextRow(); ImGui::TableNextColumn();
-        unsigned res_width  = ImGui::GetIO().DisplaySize.x;
-        unsigned res_height = ImGui::GetIO().DisplaySize.y;
+        const auto res  = ImGui::GetIO().DisplaySize;
         ImGui::PushFont(HUDElements.sw_stats->font1);
         ImGui::TextColored(HUDElements.colors.engine, "Resolution");
-        ImGui::TableNextColumn();
-        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width * 1.3, "%ix%i", res_width, res_height);
+        ImGuiTableSetColumnIndex(HUDElements.text_column);
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width * 1.3, "%.0fx%.0f", res.x, res.y);
         ImGui::PopFont();
     }
 }
@@ -638,7 +653,7 @@ void HudElements::show_fps_limit(){
         ImGui::TableNextRow(); ImGui::TableNextColumn();
         ImGui::PushFont(HUDElements.sw_stats->font1);
         ImGui::TextColored(HUDElements.colors.engine, "%s","FPS limit");
-        ImGui::TableNextColumn();
+        ImGuiTableSetColumnIndex(HUDElements.text_column);
         right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", fps);
         ImGui::PopFont();
     }
@@ -678,7 +693,7 @@ void HudElements::gamemode(){
         ImGui::TableNextRow(); ImGui::TableNextColumn();
         ImGui::PushFont(HUDElements.sw_stats->font1);
         ImGui::TextColored(HUDElements.colors.engine, "%s", "GAMEMODE");
-        ImGui::TableNextColumn();
+        ImGuiTableSetColumnIndex(HUDElements.text_column);
         right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", HUDElements.gamemode_bol ? "ON" : "OFF");
         ImGui::PopFont();
     }
@@ -689,7 +704,7 @@ void HudElements::vkbasalt(){
         ImGui::TableNextRow(); ImGui::TableNextColumn();
         ImGui::PushFont(HUDElements.sw_stats->font1);
         ImGui::TextColored(HUDElements.colors.engine, "%s", "VKBASALT");
-        ImGui::TableNextColumn();
+        ImGuiTableSetColumnIndex(HUDElements.text_column);
         right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", HUDElements.vkbasalt_bol ? "ON" : "OFF");
         ImGui::PopFont();
     }
@@ -724,7 +739,7 @@ void HudElements::battery(){
                 ImGui::Text("%%");
             }
             if (Battery_Stats.current_watt != 0) {
-                ImGui::TableNextColumn();
+                ImguiNextColumnOrNewRow();
                 right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%.1f", Battery_Stats.current_watt);
                 ImGui::SameLine(0,1.0f);
                 ImGui::PushFont(HUDElements.sw_stats->font1);
@@ -741,12 +756,12 @@ void HudElements::battery(){
                 ImGui::PushFont(HUDElements.sw_stats->font1);
                 ImGui::TextColored(HUDElements.colors.text, "%s", "Remaining Time");
                 ImGui::PopFont();
-                ImGui::TableNextColumn(); ImGui::TableNextColumn();
+                ImGuiTableSetColumnIndex(2);
                 // ImGui::TextColored(HUDElements.colors.text, "%02.0f:%02.0f:%02.0f", hours, minutes, seconds);
                 right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%02.0f:%02.0f", hours, minutes);
             }
             else {
-                ImGui::TableNextColumn();
+                ImguiNextColumnOrNewRow();
                 right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_PLUG);
             }
         }
@@ -771,9 +786,9 @@ void HudElements::gamescope_fsr(){
         ImGui::TextColored(HUDElements.colors.engine, "%s", "FSR");
         ImGui::TableNextColumn();
         right_aligned_text(FSR_COLOR, HUDElements.ralign_width, "%s", FSR_TEXT.c_str());
-        ImGui::TableNextColumn();
         if (g_fsrUpscale){
             if (!HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_hide_fsr_sharpness]) {
+                ImguiNextColumnOrNewRow();
                 right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%i", g_fsrSharpness);
                 ImGui::SameLine(0,1.0f);
                 ImGui::PushFont(HUDElements.sw_stats->font1);
@@ -850,53 +865,52 @@ void HudElements::gamescope_frame_timing(){
 void HudElements::gamepad_battery()
 {
 #ifdef __linux__
-    gamepad_update();
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gamepad_battery]) {
+        if (gamepad_found) {
+            for (int i = 0; i < gamepad_count; i++) {
+                std::string battery = gamepad_data[i].battery;
+                std::string name = gamepad_data[i].name;
+                std::string battery_percent = gamepad_data[i].battery_percent;
+                bool report_percent = gamepad_data[i].report_percent;
+                bool charging = gamepad_data[i].is_charging;
 
-    if (gamepad_found) {
-        gamepad_info();
-        for (int i = 0; i < gamepad_count; i++) {
-            std::string battery = gamepad_data[i].battery;
-            std::string name = gamepad_data[i].name;
-            std::string battery_percent = gamepad_data[i].battery_percent;
-            bool report_percent = gamepad_data[i].report_percent;
-            bool charging = gamepad_data[i].is_charging;
-
-            ImGui::TableNextRow(); ImGui::TableNextColumn();
-            ImGui::PushFont(HUDElements.sw_stats->font1);
-            ImGui::TextColored(HUDElements.colors.engine, "%s", name.c_str());
-            ImGui::TableNextColumn();
-            if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gamepad_battery_icon]) {
-                if (charging)
-                    right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_USB);
-                else {
-                    if (battery == "Full")
-                        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_BATTERY_FULL);
-                    else if (battery == "High")
-                        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_BATTERY_THREE_QUARTERS);
-                    else if (battery == "Normal")
-                        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_BATTERY_HALF);
-                    else if (battery == "Low")
-                        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_BATTERY_QUARTER);
-                    else if (battery == "Unknown")
+                ImGui::TableNextRow(); ImGui::TableNextColumn();
+                ImGui::PushFont(HUDElements.sw_stats->font1);
+                ImGui::TextColored(HUDElements.colors.engine, "%s", name.c_str());
+                ImGui::TableNextColumn();
+                if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_gamepad_battery_icon]) {
+                    if (charging)
                         right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_USB);
-                }
-            }
-            else {
-                if (charging)
-                    right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_USB);
-                else if (report_percent) {
-                    right_aligned_text(HUDElements.colors.text,HUDElements.ralign_width, "%s", battery_percent.c_str());
-                    ImGui::SameLine(0,1.0f);
-                    ImGui::Text("%%");
+                    else {
+                        if (battery == "Full")
+                            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_BATTERY_FULL);
+                        else if (battery == "High")
+                            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_BATTERY_THREE_QUARTERS);
+                        else if (battery == "Normal")
+                            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_BATTERY_HALF);
+                        else if (battery == "Low")
+                            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_BATTERY_QUARTER);
+                        else if (battery == "Unknown")
+                            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_USB);
+                    }
                 }
                 else {
-                    if (battery == "Unknown")
+                    if (charging)
                         right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_USB);
-                    else
-                        right_aligned_text(HUDElements.colors.text,HUDElements.ralign_width, "%s", battery.c_str());
+                    else if (report_percent) {
+                        right_aligned_text(HUDElements.colors.text,HUDElements.ralign_width, "%s", battery_percent.c_str());
+                        ImGui::SameLine(0,1.0f);
+                        ImGui::Text("%%");
+                    }
+                    else {
+                        if (battery == "Unknown")
+                            right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%s", ICON_FK_USB);
+                        else
+                            right_aligned_text(HUDElements.colors.text,HUDElements.ralign_width, "%s", battery.c_str());
+                    }
                 }
+                ImGui::PopFont();
             }
-            ImGui::PopFont();
         }
     }
 #endif
@@ -908,8 +922,8 @@ void HudElements::frame_count(){
         ImGui::TableNextColumn();
         ImGui::PushFont(HUDElements.sw_stats->font1);
         ImGui::TextColored(HUDElements.colors.engine, "Frame Count");
-        ImGui::TableNextColumn();
-        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%llu", HUDElements.sw_stats->n_frames);
+        ImGuiTableSetColumnIndex(HUDElements.text_column);
+        right_aligned_text(HUDElements.colors.text, HUDElements.ralign_width, "%" PRIu64, HUDElements.sw_stats->n_frames);
         ImGui::PopFont();
     }
 }
@@ -1050,12 +1064,12 @@ void HudElements::graphs(){
         ImGui::PlotLines("", arr.data(),
                 arr.size(), 0,
                 NULL, HUDElements.min, HUDElements.max,
-                ImVec2(ImGui::GetContentRegionAvailWidth() * HUDElements.params->table_columns, 50));
+                ImVec2(ImGui::GetWindowContentRegionWidth(), 50));
     } else {
         ImGui::PlotHistogram("", arr.data(),
-        arr.size(), 0,
-        NULL, HUDElements.min, HUDElements.max,
-        ImVec2(ImGui::GetContentRegionAvailWidth() * HUDElements.params->table_columns, 50));
+            arr.size(), 0,
+            NULL, HUDElements.min, HUDElements.max,
+            ImVec2(ImGui::GetWindowContentRegionWidth(), 50));
     }
     ImGui::Dummy(ImVec2(0.0f,5.0f));
     ImGui::PopStyleColor(1);
