@@ -7,6 +7,8 @@
 #include "file_utils.h"
 #include "string_utils.h"
 
+using namespace std;
+
 string os, cpu, gpu, ram, kernel, driver, cpusched;
 bool sysInfoFetched = false;
 double fps;
@@ -60,8 +62,8 @@ void writeSummary(string filename){
   auto& logArray = logger->get_log_data();
   filename = filename.substr(0, filename.size() - 4);
   filename += "_summary.csv";
-  printf("%s\n", filename.c_str());
-  SPDLOG_DEBUG("Writing summary log file [{}]", filename, logArray.size());
+  SPDLOG_INFO("{}", filename);
+  SPDLOG_DEBUG("Writing summary log file [{}]", filename);
   std::ofstream out(filename, ios::out | ios::app);
   if (out){
     out << "0.1% Min FPS," << "1% Min FPS," << "97% Percentile FPS," << "Average FPS," << "GPU Load," << "CPU Load" << "\n";
@@ -100,8 +102,9 @@ void writeSummary(string filename){
     result = total_cpu / sorted.size();
     out << result;
   } else {
-    printf("MANGOHUD: Failed to write log file\n");
+    SPDLOG_ERROR("Failed to write log file");
   }
+  out.close();
 }
 
 void writeFile(string filename){
@@ -109,27 +112,39 @@ void writeFile(string filename){
   SPDLOG_DEBUG("Writing log file [{}], {} entries", filename, logArray.size());
   std::ofstream out(filename, ios::out | ios::app);
   if (out){
-  out << "os," << "cpu," << "gpu," << "ram," << "kernel," << "driver," << "cpuscheduler" << endl;
-  out << os << "," << cpu << "," << gpu << "," << ram << "," << kernel << "," << driver << "," << cpusched << endl;
-  out << "fps," << "frametime," << "cpu_load," << "gpu_load," << "cpu_temp," << "gpu_temp," << "gpu_core_clock," << "gpu_mem_clock," << "gpu_vram_used," << "gpu_power," << "ram_used," << "elapsed" << endl;
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_log_versioning]){
+      printf("log versioning");
+      out << "v1" << endl;
+      out << MANGOHUD_VERSION << endl;
+      out << "---------------------SYSTEM INFO---------------------" << endl;
+    }
 
-  for (size_t i = 0; i < logArray.size(); i++){
-    out << logArray[i].fps << ",";
-    out << logArray[i].frametime << ",";
-    out << logArray[i].cpu_load << ",";
-    out << logArray[i].gpu_load << ",";
-    out << logArray[i].cpu_temp << ",";
-    out << logArray[i].gpu_temp << ",";
-    out << logArray[i].gpu_core_clock << ",";
-    out << logArray[i].gpu_mem_clock << ",";
-    out << logArray[i].gpu_vram_used << ",";
-    out << logArray[i].gpu_power << ",";
-    out << logArray[i].ram_used << ",";
-    out << std::chrono::duration_cast<std::chrono::nanoseconds>(logArray[i].previous).count() << "\n";
-  }
+    out << "os," << "cpu," << "gpu," << "ram," << "kernel," << "driver," << "cpuscheduler" << endl;
+    out << os << "," << cpu << "," << gpu << "," << ram << "," << kernel << "," << driver << "," << cpusched << endl;
+
+    if (HUDElements.params->enabled[OVERLAY_PARAM_ENABLED_log_versioning])
+      out << "--------------------FRAME METRICS--------------------" << endl;
+
+    out << "fps," << "frametime," << "cpu_load," << "gpu_load," << "cpu_temp," << "gpu_temp," << "gpu_core_clock," << "gpu_mem_clock," << "gpu_vram_used," << "gpu_power," << "ram_used," << "elapsed" << endl;
+
+    for (size_t i = 0; i < logArray.size(); i++){
+      out << logArray[i].fps << ",";
+      out << logArray[i].frametime << ",";
+      out << logArray[i].cpu_load << ",";
+      out << logArray[i].gpu_load << ",";
+      out << logArray[i].cpu_temp << ",";
+      out << logArray[i].gpu_temp << ",";
+      out << logArray[i].gpu_core_clock << ",";
+      out << logArray[i].gpu_mem_clock << ",";
+      out << logArray[i].gpu_vram_used << ",";
+      out << logArray[i].gpu_power << ",";
+      out << logArray[i].ram_used << ",";
+      out << std::chrono::duration_cast<std::chrono::nanoseconds>(logArray[i].previous).count() << "\n";
+    }
   } else {
-    printf("MANGOHUD: Failed to write log file\n");
+    SPDLOG_ERROR("Failed to write log file");
   }
+  out.close();
 }
 
 string get_log_suffix(){
@@ -142,7 +157,9 @@ string get_log_suffix(){
 }
 
 Logger::Logger(const overlay_params* in_params)
-  : m_params(in_params),
+  : output_folder(in_params->output_folder),
+	log_interval(in_params->log_interval),
+	log_duration(in_params->log_duration),
     m_logging_on(false),
     m_values_valid(false)
 {
@@ -155,7 +172,7 @@ void Logger::start_logging() {
   m_values_valid = false;
   m_logging_on = true;
   m_log_start = Clock::now();
-  if((!m_params->output_folder.empty()) && (m_params->log_interval != 0)){
+  if(log_interval != 0){
     std::thread(&Logger::logging, this).detach();
   }
 }
@@ -167,11 +184,11 @@ void Logger::stop_logging() {
 
   calculate_benchmark_data();
 
-  if(!m_params->output_folder.empty()) {
+  if(!output_folder.empty()) {
     std::string program = get_wine_exe_name();
     if (program.empty())
         program = get_program_name();
-    m_log_files.emplace_back(m_params->output_folder + "/" + program + "_" + get_log_suffix());
+    m_log_files.emplace_back(output_folder + "/" + program + "_" + get_log_suffix());
     std::thread writefile (writeFile, m_log_files.back());
     std::thread writesummary (writeSummary, m_log_files.back());
     writefile.join();
@@ -185,13 +202,16 @@ void Logger::stop_logging() {
 #endif
   }
   clear_log_data();
+  control_client_check(HUDElements.params->control, global_control_client, gpu.c_str());
+  const char * cmd = "LoggingFinished";
+  control_send(global_control_client, cmd, strlen(cmd), 0, 0);
 }
 
 void Logger::logging(){
   wait_until_data_valid();
   while (is_active()){
       try_log();
-      this_thread::sleep_for(chrono::milliseconds(m_params->log_interval));
+      this_thread::sleep_for(std::chrono::milliseconds(log_interval));
   }
 }
 
@@ -206,7 +226,7 @@ void Logger::try_log() {
   currentLogData.frametime = frametime;
   m_log_array.push_back(currentLogData);
 
-  if(m_params->log_duration && (elapsedLog >= std::chrono::seconds(m_params->log_duration))){
+  if(log_duration && (elapsedLog >= std::chrono::seconds(log_duration))){
     stop_logging();
   }
 }
@@ -238,7 +258,10 @@ void autostart_log(int sleep) {
 }
 
 void Logger::calculate_benchmark_data(){
-  vector<float> sorted = benchmark.fps_data;
+  vector<float> sorted {};
+  for (auto& point : m_log_array)
+    sorted.push_back(point.fps);
+
   std::sort(sorted.begin(), sorted.end());
   benchmark.percentile_data.clear();
 
@@ -250,7 +273,7 @@ void Logger::calculate_benchmark_data(){
   size_t max_label_size = 0;
 
   float result;
-  for (std::string percentile : m_params->benchmark_percentiles) {
+  for (std::string percentile : HUDElements.params->benchmark_percentiles) {
       // special case handling for a mean-based average
       if (percentile == "AVG") {
         result = benchmark.total / sorted.size();
