@@ -15,7 +15,15 @@ struct memory_information {
   unsigned long long bufmem, buffers, cached;
 };
 
+struct process_mem
+{
+    int64_t virt, resident, shared;
+    int64_t text, data, dirty;
+};
+extern process_mem proc_mem;
+
 void update_meminfo(void);
+void update_procmem();
 FILE *open_file(const char *file, int *reported);
 
 #endif //MANGOHUD_MEMORY_H
