@@ -14,8 +14,10 @@
 #include "gpu.h"
 #include "memory.h"
 #include "timing.hpp"
+#include "fcat.h"
 #include "mesa/util/macros.h"
 #include "battery.h"
+#include "gamepad.h"
 #include "string_utils.h"
 #include "file_utils.h"
 #include "pci_ids.h"
@@ -29,14 +31,16 @@
 #endif
 
 namespace fs = ghc::filesystem;
+using namespace std;
 
 #ifdef HAVE_DBUS
 float g_overflow = 50.f /* 3333ms * 0.5 / 16.6667 / 2 (to edge and back) */;
 #endif
 
 string gpuString,wineVersion,wineProcess;
-int32_t deviceID;
+uint32_t deviceID;
 bool gui_open = false;
+bool fcat_open = false;
 struct benchmark_stats benchmark;
 struct fps_limit fps_limit_stats {};
 ImVec2 real_font_size;
@@ -48,6 +52,7 @@ bool gpu_metrics_exists = false;
 bool steam_focused = false;
 vector<float> frametime_data(200,0.f);
 int fan_speed;
+fcatoverlay fcatstatus;
 
 void init_spdlog()
 {
@@ -72,6 +77,20 @@ void init_spdlog()
    spdlog::set_level(spdlog::level::level_enum::debug);
 #endif
    spdlog::cfg::load_env_levels();
+
+   // Use MANGOHUD_LOG_LEVEL to correspond to SPDLOG_LEVEL
+   if (getenv("MANGOHUD_LOG_LEVEL")) {
+      std::string log_level = getenv("MANGOHUD_LOG_LEVEL");
+      vector<string> levels;
+      levels = {"off","info","err","debug"};
+      for (auto & element : levels) {
+         transform(log_level.begin(), log_level.end(), log_level.begin(), ::tolower);
+         if(log_level == element ) {
+            spdlog::set_level(spdlog::level::from_str(log_level));
+         }
+      }
+   }
+
 }
 
 void FpsLimiter(struct fps_limit& stats){
@@ -101,8 +120,8 @@ void update_hw_info(const struct overlay_params& params, uint32_t vendorID)
 #endif
    }
    if (params.enabled[OVERLAY_PARAM_ENABLED_gpu_stats] || logger->is_active()) {
-      if (vendorID == 0x1002 && getAmdGpuInfo_actual)
-         getAmdGpuInfo_actual();
+      if (vendorID == 0x1002)
+         getAmdGpuInfo();
 
       if (gpu_metrics_exists)
          amdgpu_get_metrics();
@@ -114,6 +133,12 @@ void update_hw_info(const struct overlay_params& params, uint32_t vendorID)
 #ifdef __linux__
    if (params.enabled[OVERLAY_PARAM_ENABLED_battery])
       Battery_Stats.update();
+   if (params.enabled[OVERLAY_PARAM_ENABLED_gamepad_battery]) {
+      gamepad_update();
+      if (gamepad_found) {
+            gamepad_info();
+      }
+   }
    if (params.enabled[OVERLAY_PARAM_ENABLED_ram] || params.enabled[OVERLAY_PARAM_ENABLED_swap] || logger->is_active())
       update_meminfo();
    if (params.enabled[OVERLAY_PARAM_ENABLED_procmem])
@@ -138,7 +163,7 @@ void update_hw_info(const struct overlay_params& params, uint32_t vendorID)
    if (graph_data.size() >= kMaxGraphEntries)
       graph_data.pop_front();
    graph_data.push_back(currentLogData);
-   logger->notify_data_valid();
+   if (logger) logger->notify_data_valid();
    HUDElements.update_exec();
 }
 
@@ -274,6 +299,18 @@ float get_time_stat(void *_data, int _idx)
    return data->frames_stats[idx].stats[data->stat_selector] / data->time_dividor;
 }
 
+void overlay_new_frame(const struct overlay_params& params)
+{
+   ImGui::PushStyleVar(ImGuiStyleVar_WindowBorderSize, 0.0f);
+   ImGui::PushStyleVar(ImGuiStyleVar_ItemSpacing, ImVec2(8,-3));
+   ImGui::PushStyleVar(ImGuiStyleVar_Alpha, params.alpha);
+}
+
+void overlay_end_frame()
+{
+   ImGui::PopStyleVar(3);
+}
+
 void position_layer(struct swapchain_stats& data, const struct overlay_params& params, const ImVec2& window_size)
 {
    unsigned width = ImGui::GetIO().DisplaySize.x;
@@ -284,9 +321,6 @@ void position_layer(struct swapchain_stats& data, const struct overlay_params& p
 
    ImGui::SetNextWindowBgAlpha(params.background_alpha);
    ImGui::SetNextWindowSize(window_size, ImGuiCond_Always);
-   ImGui::PushStyleVar(ImGuiStyleVar_WindowBorderSize, 0.0f);
-   ImGui::PushStyleVar(ImGuiStyleVar_ItemSpacing, ImVec2(8,-3));
-   ImGui::PushStyleVar(ImGuiStyleVar_Alpha, params.alpha);
    switch (params.position) {
    case LAYER_POSITION_TOP_LEFT:
       data.main_window_pos = ImVec2(margin + params.offset_x, margin + params.offset_y);
@@ -537,13 +571,14 @@ void render_imgui(swapchain_stats& data, struct overlay_params& params, ImVec2&
 
    if (!params.no_display && !steam_focused){
       ImGui::Begin("Main", &gui_open, ImGuiWindowFlags_NoDecoration);
-      ImGui::BeginTable("hud", params.table_columns, ImGuiTableFlags_NoClip);
-      HUDElements.place = 0;
-      for (auto& func : HUDElements.ordered_functions){
-         func.first();
-         HUDElements.place += 1;
+      if (ImGui::BeginTable("hud", params.table_columns, ImGuiTableFlags_NoClip)) {
+         HUDElements.place = 0;
+         for (auto& func : HUDElements.ordered_functions){
+            func.first();
+            HUDElements.place += 1;
+         }
+         ImGui::EndTable();
       }
-      ImGui::EndTable();
 
       if(logger->is_active())
          ImGui::GetWindowDrawList()->AddCircleFilled(ImVec2(data.main_window_pos.x + window_size.x - 15, data.main_window_pos.y + 15), 10, params.engine_color, 20);
@@ -552,6 +587,22 @@ void render_imgui(swapchain_stats& data, struct overlay_params& params, ImVec2&
       if((now - logger->last_log_end()) < 12s && !logger->is_active())
          render_benchmark(data, params, window_size, height, now);
    }
+
+   if(params.enabled[OVERLAY_PARAM_ENABLED_fcat])
+     {
+       fcatstatus.update(&params);
+       auto window_corners = fcatstatus.get_overlay_corners();
+       auto p_min=window_corners[0];
+       auto p_max=window_corners[1];
+       auto window_size= window_corners[2];
+       ImGui::SetNextWindowPos(p_min, ImGuiCond_Always);
+       ImGui::PushStyleVar(ImGuiStyleVar_WindowPadding, ImVec2(0,0));
+       ImGui::SetNextWindowSize(window_size);
+       ImGui::Begin("FCAT", &fcat_open, ImGuiWindowFlags_NoDecoration| ImGuiWindowFlags_NoBackground);
+       ImGui::GetWindowDrawList()->AddRectFilled(p_min,p_max,fcatstatus.get_next_color(data),0.0);
+       ImGui::End();
+       ImGui::PopStyleVar();
+     }
 }
 
 void init_cpu_stats(overlay_params& params)
@@ -624,23 +675,31 @@ void init_gpu_stats(uint32_t& vendorID, uint32_t reported_deviceID, overlay_para
        || gpu.find("AMD") != std::string::npos) {
       string path;
       string drm = "/sys/class/drm/";
-      getAmdGpuInfo_actual = getAmdGpuInfo;
 
       auto dirs = ls(drm.c_str(), "card");
       for (auto& dir : dirs) {
          path = drm + dir;
 
-         SPDLOG_DEBUG("amdgpu path check: {}/device/vendor", path);
+         SPDLOG_DEBUG("amdgpu path check: {}", path);
+         if (pci_bus_parsed && pci_dev) {
+            string pci_device = read_symlink((path + "/device").c_str());
+            SPDLOG_DEBUG("PCI device symlink: '{}'", pci_device);
+            if (!ends_with(pci_device, pci_dev)) {
+               SPDLOG_DEBUG("skipping GPU, no PCI ID match");
+               continue;
+            }
+         }
+
          FILE *fp;
          string device = path + "/device/device";
          if ((fp = fopen(device.c_str(), "r"))){
             uint32_t temp = 0;
             if (fscanf(fp, "%x", &temp) == 1) {
-            // if (temp != reported_deviceID && deviceID != 0){
-            //    fclose(fp);
-            //    SPDLOG_DEBUG("DeviceID does not match vulkan report {}", reported_deviceID);
-            //    continue;
-            // }
+               if (!pci_bus_parsed && reported_deviceID && temp != reported_deviceID){
+                  fclose(fp);
+                  SPDLOG_DEBUG("DeviceID does not match vulkan report {:X}", reported_deviceID);
+                  continue;
+               }
                deviceID = temp;
             }
             fclose(fp);
@@ -656,15 +715,6 @@ void init_gpu_stats(uint32_t& vendorID, uint32_t reported_deviceID, overlay_para
             fclose(fp);
          }
 
-         if (pci_bus_parsed && pci_dev) {
-            string pci_device = read_symlink((path + "/device").c_str());
-            SPDLOG_DEBUG("PCI device symlink: '{}'", pci_device);
-            if (!ends_with(pci_device, pci_dev)) {
-               SPDLOG_DEBUG("skipping GPU, no PCI ID match");
-               continue;
-            }
-         }
-
          const std::string device_path = path + "/device";
          const std::string gpu_metrics_path = device_path + "/gpu_metrics";
          if (amdgpu_check_metrics(gpu_metrics_path)) {
@@ -780,7 +830,8 @@ void init_system_info(){
             if (wine_env)
                unsetenv("WINELOADERNOEXEC");
             wineVersion = exec(findVersion.str());
-            std::cout << "WINE VERSION = " << wineVersion << "\n";
+            trim(wineVersion);
+            SPDLOG_DEBUG("WINE version: {}", wineVersion);
             if (wine_env)
                setenv("WINELOADERNOEXEC", wine_env, 1);
          }
@@ -813,7 +864,7 @@ void init_system_info(){
 #endif
 }
 
-std::string get_device_name(int32_t vendorID, int32_t deviceID)
+std::string get_device_name(uint32_t vendorID, uint32_t deviceID)
 {
    string desc;
 #ifdef __linux__
@@ -845,7 +896,7 @@ void update_fan(){
          break;
       }
    }
-   
+
    if (!hwmon_path.empty())
       fan_speed = stoi(read_line(hwmon_path));
    else
