@@ -96,9 +96,15 @@ parse_position(const char *str)
 static int
 parse_control(const char *str)
 {
-   int ret = os_socket_listen_abstract(str, 1);
+   std::string path(str);
+   size_t npos = path.find("%p");
+   if (npos != std::string::npos)
+      path.replace(npos, 2, std::to_string(getpid()));
+   SPDLOG_DEBUG("Socket: {}", path);
+
+   int ret = os_socket_listen_abstract(path.c_str(), 1);
    if (ret < 0) {
-      SPDLOG_ERROR("Couldn't create socket pipe at '{}'\n", str);
+      SPDLOG_ERROR("Couldn't create socket pipe at '{}'", path);
       SPDLOG_ERROR("ERROR: '{}'", strerror(errno));
       return ret;
    }
@@ -402,6 +408,8 @@ parse_gl_size_query(const char *str)
 #define parse_gl_bind_framebuffer(s) parse_unsigned(s)
 #define parse_gl_dont_flip(s) parse_unsigned(s) != 0
 #define parse_round_corners(s) parse_unsigned(s)
+#define parse_fcat_overlay_width(s) parse_unsigned(s)
+#define parse_fcat_screen_edge(s) parse_unsigned(s)
 
 #define parse_cpu_color(s) parse_color(s)
 #define parse_gpu_color(s) parse_color(s)
@@ -527,6 +535,7 @@ parse_overlay_env(struct overlay_params *params,
          params->enabled[OVERLAY_PARAM_ENABLED_hide_fsr_sharpness] = 0;
          params->enabled[OVERLAY_PARAM_ENABLED_throttling_status] = 0;
          params->enabled[OVERLAY_PARAM_ENABLED_read_cfg] = read_cfg;
+         params->enabled[OVERLAY_PARAM_ENABLED_fcat] = 0;
       }
 #define OVERLAY_PARAM_BOOL(name)                                       \
       if (!strcmp(#name, key)) {                                       \
@@ -576,8 +585,9 @@ parse_overlay_config(struct overlay_params *params,
    params->enabled[OVERLAY_PARAM_ENABLED_frametime] = true;
    params->enabled[OVERLAY_PARAM_ENABLED_fps_only] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_gamepad_battery] = false;
-   params->enabled[OVERLAY_PARAM_ENABLED_gamepad_battery_icon] = true;
+   params->enabled[OVERLAY_PARAM_ENABLED_gamepad_battery_icon] = false;
    params->enabled[OVERLAY_PARAM_ENABLED_throttling_status] = false;
+   params->enabled[OVERLAY_PARAM_ENABLED_fcat] = false;
    params->fps_sampling_period = 500000000; /* 500ms */
    params->width = 0;
    params->height = 140;
@@ -589,6 +599,8 @@ parse_overlay_config(struct overlay_params *params,
    params->offset_y = 0;
    params->background_alpha = 0.5;
    params->alpha = 1.0;
+   params->fcat_screen_edge = 0;
+   params->fcat_overlay_width = 24;
    params->time_format = "%T";
    params->gpu_color = 0x2e9762;
    params->cpu_color = 0x2e97cb;
@@ -669,6 +681,7 @@ parse_overlay_config(struct overlay_params *params,
          params->enabled[OVERLAY_PARAM_ENABLED_mangoapp_steam] = 0;
          params->enabled[OVERLAY_PARAM_ENABLED_hide_fsr_sharpness] = 0;
          params->enabled[OVERLAY_PARAM_ENABLED_throttling_status] = 0;
+         params->enabled[OVERLAY_PARAM_ENABLED_fcat] = 0;
          params->options.erase("full");
       }
       for (auto& it : params->options) {
@@ -699,6 +712,10 @@ parse_overlay_config(struct overlay_params *params,
       parse_overlay_env(params, env);
    }
 
+   // If fps_only param is enabled disable legacy_layout
+   if (params->enabled[OVERLAY_PARAM_ENABLED_fps_only])
+      params->enabled[OVERLAY_PARAM_ENABLED_legacy_layout] = false;
+
    if (is_blacklisted())
       return;
 
@@ -741,17 +758,20 @@ parse_overlay_config(struct overlay_params *params,
    if (!params->table_columns)
       params->table_columns = 3;
 
+   params->table_columns = std::max(1u, std::min(64u, params->table_columns));
+
    if (!params->font_size) {
       params->font_size = 24;
    }
 
    //increase hud width if io read and write
    if (!params->width) {
+      params->width = params->font_size * params->font_scale * params->table_columns * 4;
+
       if ((params->enabled[OVERLAY_PARAM_ENABLED_io_read] || params->enabled[OVERLAY_PARAM_ENABLED_io_write])) {
-         params->width = 13 * params->font_size * params->font_scale;
-      } else {
-         params->width = params->font_size * params->font_scale * 11.7;
+         params->width += 2 * params->font_size * params->font_scale;
       }
+
       // Treat it like hud would need to be ~7 characters wider with default font.
       if (params->no_small_font)
          params->width += 7 * params->font_size * params->font_scale;
@@ -810,9 +830,16 @@ parse_overlay_config(struct overlay_params *params,
 
    // Needs ImGui context but it is null here for OpenGL so just note it and update somewhere else
    HUDElements.colors.update = true;
+   if (params->no_small_font)
+      HUDElements.text_column = 2;
+   else
+      HUDElements.text_column = 1;
 
-   if(logger && logger->m_params == nullptr) logger.reset();
-   if(!logger) logger = std::make_unique<Logger>(HUDElements.params);
+   if(logger && logger->is_active()){
+      SPDLOG_DEBUG("Stopped logging because config reloaded");
+      logger->stop_logging();
+   }
+   logger = std::make_unique<Logger>(params);
    if(params->autostart_log && !logger->is_active())
       std::thread(autostart_log, params->autostart_log).detach();
 #ifdef MANGOAPP
