@@ -14,11 +14,12 @@ void init_d3d_shared(){
     if (cfg_inited)
         return;
      parse_overlay_config(&params, getenv("MANGOHUD_CONFIG"));
+     _params = &params;
      cfg_inited = true;
     //  init_cpu_stats(params);
 }
 
 void d3d_run(){
-    check_keybinds(sw_stats, params, vendorID);
+    check_keybinds(params, vendorID);
 	update_hud_info(sw_stats, params, vendorID);
 }
\ No newline at end of file
